@@ -81,11 +81,14 @@ from .utils.logging import WebhookFormatter, WebhookHandler
 from .utils.request_handler import NotFoundHandler
 from .utils.static_file_handling import StaticFileHandler
 from .utils.utils import (
+    BetterConfigParser,
     Handler,
     ModuleInfo,
     Permission,
     Timer,
     geoip,
+    parse_command_line_arguments,
+    parse_config,
     time_function,
 )
 
@@ -95,9 +98,6 @@ IGNORED_MODULES: Final[set[str]] = {
     "templates.*",
 } | (set() if sys.flags.dev_mode else {"example.*"})
 
-CONFIG: Final[ConfigParser] = ConfigParser(interpolation=None)
-CONFIG.read("config.ini", encoding="UTF-8")
-
 LOGGER: Final = logging.getLogger(__name__)
 
 HEARTBEAT: float = 0
@@ -304,7 +304,11 @@ def get_all_handlers(module_infos: tuple[ModuleInfo, ...]) -> list[Handler]:
     handlers.append((r"(?i)/api/*", RedirectHandler, {"url": "/api/endpunkte"}))
 
     handlers.append(
-        (r"(?i)/\.well-known/(.*)", StaticFileHandler, {"path": ".well-known"})
+        (
+            r"(?i)/\.well-known/(.*)",
+            StaticFileHandler,
+            {"path": ".well-known", "keep_case": True},
+        )
     )
 
     LOGGER.debug("Loaded %d handlers", len(handlers))
@@ -312,14 +316,11 @@ def get_all_handlers(module_infos: tuple[ModuleInfo, ...]) -> list[Handler]:
     return handlers
 
 
-def ignore_modules(config: ConfigParser) -> None:
+def ignore_modules(config: BetterConfigParser) -> None:
     """Read ignored modules from the config."""
-    for module_name in config.get(
-        "GENERAL", "IGNORED_MODULES", fallback=""
-    ).split(","):
-        module_name = module_name.strip()  # pylint: disable=redefined-loop-name
-        if len(module_name) > 0:
-            IGNORED_MODULES.add(module_name)
+    IGNORED_MODULES.update(
+        config.getset("GENERAL", "IGNORED_MODULES", fallback="")
+    )
 
 
 def make_app(config: ConfigParser) -> str | Application:
@@ -351,12 +352,12 @@ def make_app(config: ConfigParser) -> str | Application:
     )
 
 
-def apply_config_to_app(app: Application, config: ConfigParser) -> None:
+def apply_config_to_app(app: Application, config: BetterConfigParser) -> None:
     """Apply the config (from the config.ini file) to the application."""
     app.settings["CONFIG"] = config
 
     app.settings["cookie_secret"] = config.get(
-        "GENERAL", "COOKIE_SECRET", fallback=b"xyzzy"
+        "GENERAL", "COOKIE_SECRET", fallback="xyzzy"
     )
 
     app.settings["CRAWLER_SECRET"] = config.get(
@@ -412,9 +413,9 @@ def apply_config_to_app(app: Application, config: ConfigParser) -> None:
             if len(key_perms) > 1
             else (1 << len(Permission)) - 1  # should be all permissions
         )
-        for secret in config.get(
+        for secret in config.getset(
             "GENERAL", "TRUSTED_API_SECRETS", fallback="xyzzy"
-        ).split(",")
+        )
         if (key_perms := [part.strip() for part in secret.split("=")])
         if key_perms[0]
     }
@@ -585,7 +586,8 @@ def setup_apm(app: Application) -> None:  # pragma: no cover
             app.settings["ELASTIC_APM"]["INLINE_SCRIPT"].encode("ASCII")
         ).digest()
     ).decode("ASCII")
-    app.settings["ELASTIC_APM"]["CLIENT"] = ElasticAPM(app).client
+    if app.settings["ELASTIC_APM"]["ENABLED"]:
+        app.settings["ELASTIC_APM"]["CLIENT"] = ElasticAPM(app).client
 
 
 def setup_app_search(app: Application) -> None:  # pragma: no cover
@@ -615,15 +617,9 @@ def setup_app_search(app: Application) -> None:  # pragma: no cover
 def setup_elasticsearch(app: Application) -> None | AsyncElasticsearch:
     """Setup Elasticsearch."""  # noqa: D401
     config = app.settings["CONFIG"]
-    if not config.getboolean("ELASTICSEARCH", "ENABLED", fallback=False):
-        app.settings["ELASTICSEARCH"] = None
-        return None
-    elasticsearch = AsyncElasticsearch(
+    kwargs = dict(  # noqa: C408
         cloud_id=config.get("ELASTICSEARCH", "CLOUD_ID", fallback=None),
-        hosts=tuple(
-            host.strip()
-            for host in config.get("ELASTICSEARCH", "HOSTS").split(",")
-        )
+        hosts=tuple(config.getset("ELASTICSEARCH", "HOSTS"))
         if config.has_option("ELASTICSEARCH", "HOSTS")
         else None,
         url_prefix=config.get("ELASTICSEARCH", "URL_PREFIX", fallback=""),
@@ -649,6 +645,10 @@ def setup_elasticsearch(app: Application) -> None | AsyncElasticsearch:
             "accept": "application/vnd.elasticsearch+json; compatible-with=7"
         },
     )
+    if not config.getboolean("ELASTICSEARCH", "ENABLED", fallback=False):
+        app.settings["ELASTICSEARCH"] = None
+        return None
+    elasticsearch = AsyncElasticsearch(**kwargs)
     app.settings["ELASTICSEARCH"] = elasticsearch
     return elasticsearch
 
@@ -746,9 +746,6 @@ async def setup_elasticsearch_configs(  # noqa: C901
 def setup_redis(app: Application) -> None | Redis[str]:
     """Setup Redis."""  # noqa: D401
     config = app.settings["CONFIG"]
-    if not config.getboolean("REDIS", "ENABLED", fallback=False):
-        app.settings["REDIS"] = None
-        return None
     kwargs = {
         "db": config.getint("REDIS", "DB", fallback=0),
         "username": config.get("REDIS", "USERNAME", fallback=None),
@@ -792,6 +789,9 @@ def setup_redis(app: Application) -> None | Redis[str]:
                     ),
                 }
             )
+    if not config.getboolean("REDIS", "ENABLED", fallback=False):
+        app.settings["REDIS"] = None
+        return None
     redis: Redis[str] = Redis(connection_pool=BlockingConnectionPool(**kwargs))
     app.settings["REDIS"] = redis
     return redis
@@ -842,29 +842,40 @@ async def heartbeat() -> None:
     global HEARTBEAT  # pylint: disable=global-statement
     while True:  # pylint: disable=while-used
         HEARTBEAT = time.monotonic()
-        await asyncio.sleep(1)
+        await asyncio.sleep(0)
 
 
 def supervise(loop: AbstractEventLoop) -> None:
     """Supervise."""
     while not loop.is_closed():  # pylint: disable=while-used
         if time.monotonic() - HEARTBEAT >= 10:
+            task_id = tornado.process.task_id()
+            pid = os.getpid()
+            LOGGER.fatal(
+                "Heartbeat timed out for worker %d (pid %d)", task_id, pid
+            )
             with contextlib.suppress(OSError):
-                os.kill(os.getpid(), getattr(signal, "CTRL_BREAK_EVENT", 9))
+                os.kill(pid, getattr(signal, "CTRL_BREAK_EVENT", 9))
         time.sleep(1)
 
 
-def main() -> None | int | str:  # noqa: C901  # pragma: no cover
+def main(  # noqa: C901
+    config: BetterConfigParser | None = None,
+) -> None | int | str:  # pragma: no cover
     """
     Start everything.
 
-    This is the main function that is called when running this file.
+    This is the main function that is called when running this programm.
     """
     # pylint: disable=too-complex, too-many-branches
     # pylint: disable=too-many-locals, too-many-statements
     setproctitle(NAME)
     install_signal_handler()
-    setup_logging(CONFIG)
+
+    args = parse_command_line_arguments()
+    config = parse_config(*args.config) if config is None else config
+
+    setup_logging(config)
 
     LOGGER.info("Starting %s %s", NAME, VERSION)
 
@@ -874,42 +885,47 @@ def main() -> None | int | str:  # noqa: C901  # pragma: no cover
             NAME.removesuffix("-dev"),
         )
 
-    ignore_modules(CONFIG)
-    app = make_app(CONFIG)
+    ignore_modules(config)
+    app = make_app(config)
     if isinstance(app, str):
         return app
 
-    apply_config_to_app(app, CONFIG)
+    apply_config_to_app(app, config)
     setup_elasticsearch(app)
     setup_app_search(app)
     setup_redis(app)
 
-    if CONFIG.getboolean("ELASTIC_APM", "ENABLED", fallback=False):
-        setup_apm(app)
+    setup_apm(app)
 
-    behind_proxy = CONFIG.getboolean("GENERAL", "BEHIND_PROXY", fallback=False)
+    behind_proxy = config.getboolean("GENERAL", "BEHIND_PROXY", fallback=False)
 
     server = HTTPServer(
         app,
-        ssl_options=get_ssl_context(CONFIG),
+        body_timeout=3600,
         decompress_request=True,
+        max_body_size=1_000_000_000,
+        ssl_options=get_ssl_context(config),
         xheaders=behind_proxy,
     )
 
     sockets = []
 
-    port = CONFIG.getint(
-        "GENERAL", "PORT", fallback=8888 if CONTAINERIZED else None
-    )
+    ports = {
+        *args.port,
+        config.getint(
+            "GENERAL", "PORT", fallback=8888 if CONTAINERIZED else None
+        ),
+    }
 
-    if port:
-        sockets.extend(
-            tornado.netutil.bind_sockets(
-                port, "localhost" if behind_proxy else ""
+    for port in ports:
+        if port:
+            sockets.extend(
+                tornado.netutil.bind_sockets(
+                    port, "localhost" if behind_proxy else ""
+                )
             )
-        )
 
-    unix_socket_path = CONFIG.get(
+    unix_socket_path = config.get(
         "GENERAL",
         "UNIX_SOCKET_PATH",
         fallback="/data" if CONTAINERIZED else None,
@@ -924,17 +940,17 @@ def main() -> None | int | str:  # noqa: C901  # pragma: no cover
             )
         )
 
-    processes = CONFIG.getint(
+    processes = config.getint(
         "GENERAL",
         "PROCESSES",
-        fallback=2
-        if sys.flags.dev_mode
-        else (tornado.process.cpu_count() if hasattr(os, "fork") else 0),
+        fallback=hasattr(os, "fork") * (2 if sys.flags.dev_mode else -1),
     )
+    if processes < 0:
+        processes = tornado.process.cpu_count()
 
     task_id: int | None = None
 
-    if processes > 0:
+    if processes and sockets:
         setproctitle(f"{NAME} - Master")
 
         task_id = tornado.process.fork_processes(processes)
@@ -959,7 +975,7 @@ def main() -> None | int | str:  # noqa: C901  # pragma: no cover
 
     # get loop after forking
     loop = asyncio.get_event_loop_policy().get_event_loop()
-    setup_webhook_logging(CONFIG, loop)
+    setup_webhook_logging(config, loop)
 
     _ = asyncio.get_event_loop
     asyncio.get_event_loop = lambda: loop
@@ -968,30 +984,43 @@ def main() -> None | int | str:  # noqa: C901  # pragma: no cover
 
     # pylint: disable=unused-variable
 
-    heartbeat_task = loop.create_task(heartbeat())  # noqa: F841
+    if sockets:
+        heartbeat_task = loop.create_task(heartbeat())  # noqa: F841
 
-    wait_for_shutdown_task = loop.create_task(wait_for_shutdown())  # noqa: F841
+        wait_for_shutdown_task = loop.create_task(  # noqa: F841
+            wait_for_shutdown()
+        )
 
-    if CONFIG.getboolean("ELASTICSEARCH", "ENABLED", fallback=False):
+    if (
+        config.getboolean("ELASTICSEARCH", "ENABLED", fallback=False)
+        and sockets
+    ):
         check_es_task = loop.create_task(check_elasticsearch(app))  # noqa: F841
 
-    if CONFIG.getboolean("REDIS", "ENABLED", fallback=False):
+    if config.getboolean("REDIS", "ENABLED", fallback=False) and sockets:
         check_redis_task = loop.create_task(check_redis(app))  # noqa: F841
 
-    if not task_id:  # update from one process only
+    if not task_id and sockets:  # update from one process only
         quotes_cache_update_task = loop.create_task(  # noqa: F841
             update_cache_periodically(app)
         )
 
     # pylint: enable=unused-variable
 
-    if CONFIG.getboolean("GENERAL", "SUPERVISE", fallback=False):
+    if config.getboolean("GENERAL", "SUPERVISE", fallback=False) and sockets:
         global HEARTBEAT  # pylint: disable=global-statement
         HEARTBEAT = time.monotonic()
         threading.Thread(
             target=supervise, name="supervisor", args=(loop,), daemon=True
         ).start()
 
+    if args.save_config_to:
+        with open(args.save_config_to, "w", encoding="UTF-8") as file:
+            config.write(file, True)
+
+    if not sockets:
+        return None
+
     try:
         loop.run_forever()
         EVENT_SHUTDOWN.set()
