@@ -331,7 +331,9 @@ async def make_api_request(
             response.reason,
         )
         raise HTTPError(
-            400,
+            400
+            if response.code == 500
+            else (404 if response.code == 404 else 503),
             reason=(
                 f"{API_URL}/{endpoint} returned: "
                 f"{response.code} {response.reason}"
@@ -396,11 +398,7 @@ def parse_quote(json_data: dict[str, Any], quote: None | Quote = None) -> Quote:
         QUOTES_CACHE[quote.id] = quote
         MAX_QUOTES_ID.value = max(MAX_QUOTES_ID.value, quote.id)
     else:  # quote was already saved
-        quote.update_quote(
-            quote_str,
-            author.id,
-            author.name,
-        )
+        quote.update_quote(quote_str, author.id, author.name)
     return quote
 
 
